import 'dart:async';
import 'package:appgain_io_example/text_field_alert_dialog.dart';
import 'package:logger/logger.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:flutter/services.dart';

class Appgain {
  static const MethodChannel _channel =
  MethodChannel('com.Appgain.io/AppgainSDK');
  final _initializeAppgainSDK = 'initSDK';
  final _gettingUserId = 'getUserID';
  final _createMatchLink = 'matchLink';
  final _createFireAutomatorWithPersonalization =
      'fireAutomatorWithPersonalization';
  final _createFireAutomator = 'fireAutomator';
  final _cancelAutomator = 'cancelAutomator';
  final _setUpUserId = 'setUserId';
  final _logNewPurchase = 'addPurchase';
  final _updateChannel = 'updateChannel';
  final _addNotificationChannel = 'addNotificationChannel';
  final _updateUserWith = 'updateUser';
  final _logEventWith = 'logEvent';

  var logger = Logger(printer: PrettyPrinter());

  /// required appId and app key found it in dashboard found in
  /// (https://docs.appgain.io/basicIntegration/configureYourDashboard/#sdk-integration)
  Future initAppgainSDK({required String? appid , required String? apiKey}) async {
    var _prefs = await SharedPreferences.getInstance();
    var newAppId = _prefs.getString('appId');
    var newAppKey = _prefs.getString('appKey');
    var data = {'appId': appid, 'apiKey': apiKey, 'configure': true};
    try {
      await _channel.invokeMethod(_initializeAppgainSDK, data).then((result) {
        logger.d('initAppgainSDK $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e.toString());
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// Geting Appgain user Id
  Future getUserId() async {
    try {
      await _channel.invokeMethod(_gettingUserId, '').then((result) {
        logger.d('getUserId $result');
        TextFieldAlertDialog().showAlertDialog('userId= ${result.toString()}');
      });
    } on PlatformException catch (e) {
      logger.d(e.toString());
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  Future matchLink() async {
    try {
      await _channel.invokeMethod(_createMatchLink, '').then((result) {
        logger.d('matchLink $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// Create your auto message in Appgain Dashboard
  /// required triggerPoint => Trigger that you named in dashboard
  Future fireAutomator({required String triggerPoint}) async {
    var data = {'triggerPoint': triggerPoint};
    try {
      await _channel.invokeMethod(_createFireAutomator, data).then((result) {
        logger.d('fireAutomator $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// fire Automator with  Personalization
  /// required triggerPoint user name
  /// you can add map with personalization
  /// for Example:-
  ///    var personalizationData = {
  ///       'name': 'appgain',
  ///       'cartName': 'sport',
  ///       'value': 22,
  ///     };
  Future fireAutomatorWithPersonalization({required String? triggerPoint,
    Map<String, Object>? personalizationMap}) async {
    var data = {
      'triggerPoint': triggerPoint,
      'personalizationMap': personalizationMap
    };
    try {
      await _channel
          .invokeMethod(_createFireAutomatorWithPersonalization, data)
          .then((result) {
        logger.d('fireAutomatorWithPersonalization $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// for example --> when delete cart items you can cancel fire automator
  /// that you fire it
  Future cancelfireAutomator({required String triggerPoint}) async {
    var data = {'triggerPoint': triggerPoint};
    try {
      await _channel.invokeMethod(_cancelAutomator, data).then((result) {
        logger.d('cancelfireAutomator $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// Geting Appgain user Id
  Future setuserId({required String userId}) async {
    try {
      var data = {'userId': userId};
      await _channel.invokeMethod(_setUpUserId, data).then((result) {
        logger.d('new user id $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e.message);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// required product name, currency and amount to your product
  Future addPurchase({required String productName,
    required String currency,
    required double amount}) async {
    var data = {
      'productName': productName,
      'currency': currency,
      'amount': amount
    };
    try {
      await _channel.invokeMethod(_logNewPurchase, data).then((result) {
        logger.d('Success');
        TextFieldAlertDialog().showAlertDialog(result);
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  ///MARK: enableReciveNotification
  ///enable (String)
  ///type (String) one of theses values ("appPush","SMS","email")
  ///Output : -  callback with two parameter
  ///success bool as string
  ///error
  Future enableReciveNotification({required String? type}) async {
    try {
      var data = {'enable': true, 'type': type};
      await _channel.invokeMethod(_updateChannel, data).then((result) {
        logger.d('enableReciveNotification $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// notificationType (email or SMS)
  /// item  (user.email or phone number for SMS)
  Future addNotificationChannel(
      {required String? notificationType, required String item}) async {
    var data = {"notificationType": notificationType, "item": item};
    try {
      await _channel.invokeMethod(_addNotificationChannel, data).then((result) {
        logger.d('addNotificationChannel $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  /// required map
  /// Example:
  ///  var updatedData = {
  ///       'email': 'email',
  ///       'phone': 'phone',
  ///       'updatedField1': 'value',
  ///       "updatedField2": "value"
  ///     };
  ///
  ///
  Future updateUser({required Map<String, String> data}) async {
    var updateData = {'userData': data};
    try {
      await _channel.invokeMethod(_updateUserWith, updateData).then((result) {
        logger.d('updateUser $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  Future logEvent({required String? type,
    String? action,
    Map<String, String>? extras}) async {
    var data = {
      "action": action,
      "type": type,
      'extras': extras,
    };
    try {
      await _channel.invokeMethod(_logEventWith, data).then((result) {
        logger.d('logEvent $result');
        TextFieldAlertDialog().showAlertDialog('Success');
      });
    } on PlatformException catch (e) {
      logger.d(e);
      TextFieldAlertDialog().showAlertDialog(e.toString());
    }
  }

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }
}
